/*****************************************************************************
* OS/2  PM/C Language Demo - Drawing
* 
*       This demo is intended for comparison with the Drawing.java 
*       Magercise.
* 
* Copyright 1997, <a href=http://www.MageLang.com>MageLang Institute</a>.
******************************************************************************/
#define INCL_WIN
#define INCL_GPI

#include <os2.h>                        /* PM header file               */
#include "drawing.h"                    /* Resource symbolic identifiers*/

void DrawStickBoy(HPS hps, INT xo);

#define STRINGLENGTH 20                 /* Length of string             */

/*
 * Function prototypes
 */
MRESULT EXPENTRY MyWindowProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 );

                                        /* Define parameters by type    */
HAB  hab;                               /* PM anchor block handle       */
HWND hwndClient = NULLHANDLE;           /* Client area window handle    */
PSZ  pszErrMsg;


INT main (VOID)
{
  HMQ  hmq;                             /* Message queue handle         */
  HWND hwndFrame = NULLHANDLE;          /* Frame window handle          */
  QMSG qmsg;                            /* Message from message queue   */
  ULONG flCreate;                       /* Window creation control flags*/

  if ((hab = WinInitialize(0)) == 0L) /* Initialize PM     */
     AbortProgram(hwndFrame, hwndClient); /* Terminate the application    */

  if ((hmq = WinCreateMsgQueue( hab, 0 )) == 0L)/* Create a msg queue */
     AbortProgram(hwndFrame, hwndClient); /* Terminate the application    */

  if (!WinRegisterClass(                /* Register window class        */
     hab,                               /* Anchor block handle          */
     (PSZ)"MyWindow",                   /* Window class name            */
     (PFNWP)MyWindowProc,               /* Address of window procedure  */
     CS_SIZEREDRAW,                     /* Class style                  */
     0                                  /* No extra window words        */
     ))
     AbortProgram(hwndFrame, hwndClient); /* Terminate the application    */

   flCreate = FCF_STANDARD &            /* Set frame control flags to   */
             ~FCF_SHELLPOSITION;        /* standard except for shell    */
                                        /* positioning.                 */

  if ((hwndFrame = WinCreateStdWindow(
               HWND_DESKTOP,            /* Desktop window is parent     */
               0,                       /* STD. window styles           */
               &flCreate,               /* Frame control flag           */
               "MyWindow",              /* Client window class name     */
               "",                      /* No window text               */
               0,                       /* No special class style       */
               (HMODULE)0L,             /* Resource is in .EXE file     */
               ID_WINDOW,               /* Frame window identifier      */
               &hwndClient              /* Client window handle         */
               )) == 0L)
     AbortProgram(hwndFrame, hwndClient); /* Terminate the application    */

    WinSetWindowText(hwndFrame, "MageLang OS/2 Sample");

  if (!WinSetWindowPos( hwndFrame,      /* Shows and activates frame    */
                   HWND_TOP,            /* window at position 100, 100, */
                   100, 100, 300, 330,  /* and size 300, 330.           */
                   SWP_SIZE | SWP_MOVE | SWP_ACTIVATE | SWP_SHOW
                 ))
     AbortProgram(hwndFrame, hwndClient); /* Terminate the application    */

/*
 * Get and dispatch messages from the application message queue
 * until WinGetMsg returns FALSE, indicating a WM_QUIT message.
 */

  while( WinGetMsg( hab, &qmsg, 0L, 0, 0 ) )
    WinDispatchMsg( hab, &qmsg );
  WinDestroyWindow(hwndFrame);           /* Tidy up...                   */
  WinDestroyMsgQueue( hmq );             /* Tidy up...                   */
  WinTerminate( hab );                   /* Terminate the application    */
  return 0;
} /* End of main */



MRESULT EXPENTRY MyWindowProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  static POINTL start;
  static POINTL curr;

  switch( msg )
  {
    case WM_CREATE:
      break;
    case WM_COMMAND:
      /*
       * When Exit is chosen, the application posts itself a WM_CLOSE
       * message.
       */
      {
         USHORT command;                   /* WM_COMMAND command value     */
         command = SHORT1FROMMP(mp1);      /* Extract the command value    */
         switch (command)
         {
           case ID_EXITPROG:
             WinPostMsg( hwnd, WM_CLOSE, (MPARAM)0, (MPARAM)0 );
             break;
           default:
             return WinDefWindowProc( hwnd, msg, mp1, mp2 );
         }

         break;
      }
    case WM_PAINT:
      /*
       * Window contents are drawn here in WM_PAINT processing.
       */
      {
         HPS    hps;                       /* Presentation Space handle    */
         RECTL  rc;                        /* Rectangle coordinates        */

               			   /* Create a presentation space  */
         hps = WinBeginPaint( hwnd, 0L, &rc );
         WinFillRect( hps, &rc, SYSCLR_WINDOW);
         DrawStickBoy(hps,0);
         DrawStickBoy(hps,100);
         WinEndPaint( hps );                      /* Drawing is complete   */
         break;
      }
    case WM_ERASEBACKGROUND:
      /*
       * Return TRUE to request PM to paint the window background
       * in SYSCLR_WINDOW.
       */
      return (MRESULT)( TRUE );
    case WM_CLOSE:
      /*
       * This is the place to put termination routines
       */
      WinPostMsg( hwnd, WM_QUIT, (MPARAM)0,(MPARAM)0 );
      break;
    default:
      return WinDefWindowProc( hwnd, msg, mp1, mp2 );
  }
  return (MRESULT)FALSE;
} /* End of MyWindowProc */

VOID AbortProgram(HWND hwndFrame, HWND hwndClient)
{
   PERRINFO  pErrInfoBlk;
   PSZ       pszOffSet;
   void      stdprint(void);

   DosBeep(100,10);
   if ((pErrInfoBlk = WinGetErrorInfo(hab)) != (PERRINFO)NULL)
   {
      pszOffSet = ((PSZ)pErrInfoBlk) + pErrInfoBlk->offaoffszMsg;
      pszErrMsg = ((PSZ)pErrInfoBlk) + *((PSHORT)pszOffSet);
      if((INT)hwndFrame && (INT)hwndClient)
         WinMessageBox(HWND_DESKTOP,         /* Parent window is desk top */
                       hwndFrame,            /* Owner window is our frame */
                       (PSZ)pszErrMsg,       /* PMWIN Error message       */
                       "Error Msg",          /* Title bar message         */
                       MSGBOXID,             /* Message identifier        */
                       MB_MOVEABLE | MB_CUACRITICAL | MB_CANCEL ); /* Flags */
      WinFreeErrorInfo(pErrInfoBlk);
   }
   WinPostMsg(hwndClient, WM_QUIT, (MPARAM)NULL, (MPARAM)NULL);
} /* End */


/******************************************************************************
* Convenience function for drawing stick figure in presentation space.
******************************************************************************/
void DrawStickBoy(HPS hps, INT xo)
{
   POINTL pt;

   GpiSetColor(hps, CLR_BLUE);
   pt.x = xo + 52; pt.y = 178;
   GpiSetCurrentPosition(hps, &pt);
   pt.x += 46; pt.y -= 74;
   GpiBox(hps, DRO_FILL, &pt, 0, 0);
   GpiSetColor(hps, CLR_BLACK);

   pt.x = xo + 76; pt.y = 104;
   GpiSetCurrentPosition(hps, &pt);
   pt.x = xo + 43; pt.y = 40;
   GpiLine(hps, &pt);

   pt.x = xo + 76; pt.y = 104;
   GpiSetCurrentPosition(hps, &pt);
   pt.x = xo + 107; pt.y = 40;
   GpiLine(hps, &pt);

   pt.x = xo + 99; pt.y = 172;
   GpiSetCurrentPosition(hps, &pt);
   pt.x = xo + 134; pt.y = 200;
   GpiLine(hps, &pt);

   pt.x = xo + 52; pt.y = 172;
   GpiSetCurrentPosition(hps, &pt);
   pt.x = xo + 35; pt.y = 115;
   GpiLine(hps, &pt);

   pt.x = xo + 26; pt.y = 114;
   GpiSetCurrentPosition(hps, &pt);
   GpiFullArc(hps, DRO_OUTLINE, MAKEFIXED(8,0));
   pt.x = xo + 134; pt.y = 210;
   GpiSetCurrentPosition(hps, &pt);
   GpiFullArc(hps, DRO_OUTLINE, MAKEFIXED(8,0));
   pt.x = xo + 75; pt.y = 200;
   GpiSetCurrentPosition(hps, &pt);
   GpiFullArc(hps, DRO_OUTLINE, MAKEFIXED(16,0));

   pt.x = xo + 36; pt.y = 40;
   GpiSetCurrentPosition(hps, &pt);
   pt.x += 16; pt.y -= 9;
   GpiBox(hps, DRO_OUTLINE, &pt, 0, 0);
   pt.x = xo + 99; pt.y = 40;
   GpiSetCurrentPosition(hps, &pt);
   pt.x += 16; pt.y -= 9;
   GpiBox(hps, DRO_OUTLINE, &pt, 0, 0);

   pt.x = xo + 55; pt.y = 12;
   GpiSetCurrentPosition(hps, &pt);
   GpiCharString(hps, 9, "Stick Boy"); 
}
