/*****************************************************************************
* OS/2  PM/C Language Demo - MiniPIM                                         *
*                                                                            *
*       This demo is intended for comparison with the MiniPIM.java           *
*       Magercise.                                                           *
*                                                                            *
* Copyright 1997, <a href=http://www.MageLang.com>MageLang Institute</a>.    *
******************************************************************************/
#define INCL_WIN
#define INCL_GPI
#define INCL_DOSFILEMGR
#define INCL_DOSERRORS

#include <os2.h>                        /* PM header file               */
#include <string.h>                     /* for strlen prototype         */
#include "PIMComn.h"
#include "PIMEdit.h"                    /* Function prototypes          */

/* File Dialog constants */
#define MLE_OPEN  1
#define MLE_SAVE  2

/* local function prototypes */
static VOID SaveMLEToFile(HWND hwndMle, PCHAR pszFileName);
static VOID LoadMLEFromFile(HWND hwndMle, PCHAR pszFileName);
static BOOL DoFileDialog(HWND hwnd, INT Mode, PCHAR pchFile);
static VOID ClearMLE(HWND hwndMle);

/* module variables */
static HWND hwndMle;                    /* MLE window handle            */
static CHAR szFilename[256];

/*****************************************************/
/* Register the notepad window class                 */
/*                                                   */
/* Keep the details of the class name local to this  */
/* module so we can change them independent from the */
/* main program.                                     */
/*****************************************************/
BOOL RegisterEditClass(HAB hab)
{
   return WinRegisterClass(hab, (PSZ)"PIMEdit", (PFNWP)EditWindowProc, CS_SIZEREDRAW, 0);
}


/*****************************************************/
/* Create the notepad window                         */
/*                                                   */
/* The window is created as a child of the parent,   */
/* which is also the owner of this window.           */
/*****************************************************/
HWND CreateEditWindow(HWND hwndParent)
{
   HWND hwnd = WinCreateWindow(hwndParent,
                               "PIMEdit",
                               "",
                               WS_VISIBLE,
                               0, 0, 495, 400,
                               hwndParent,
                               HWND_TOP,
                               ID_EDITWINDOW,
                               NULL,
                               NULL);
   return hwnd;
}

/*****************************************************/
/* Notepad Window Proc                               */
/*                                                   */
/* Handle events directed to the notepad window.     */
/*****************************************************/
MRESULT EXPENTRY EditWindowProc(HWND hwndParent, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  LONG i;
  RECTL rcl;

  switch( msg )
  {
    case WM_CREATE:

       hwndMle = WinCreateWindow(
                 hwndParent,                    /* Parent window          */
                 WC_MLE,                        /* Window class           */
                 "",                            /* Window text            */
                 WS_VISIBLE |
                 MLS_BORDER |
                 MLS_VSCROLL |
                 MLS_HSCROLL,                   /* MultiLineEdit Control  */
                 5, 			        /* x position             */
                 25,			        /* y position             */
                 485,      	                /* Width                  */
                 370,                           /* Height                 */
                 hwndParent,                    /* Owner window           */
                 HWND_TOP,                      /* Top of z-order         */
                 ID_MLE,                        /* Window identifier      */
                 NULL,                          /* Control data           */
                 NULL);                         /* Presentation parameters*/
      break;
    case WM_COMMAND:
      /*
       * When Exit is chosen, the application posts itself a WM_CLOSE
       * message.
       */
      {
      USHORT command;                   /* WM_COMMAND command value     */
      command = SHORT1FROMMP(mp1);      /* Extract the command value    */
      switch (command)
      {
        case ID_NEWFILE:
          ClearMLE(hwndMle);
          strcpy(szFilename, "");
          break;
        case ID_OPENFILE:
          if (DoFileDialog(hwndParent, MLE_OPEN, szFilename))
          {
             if (strlen(szFilename) > 0)
                LoadMLEFromFile(hwndMle,szFilename);
          }
          break;
        case ID_SAVEAS:
        case ID_SAVEFILE:
          if (DoFileDialog(hwndParent, MLE_SAVE, szFilename))
          {
             if (strlen(szFilename) > 0)
                SaveMLEToFile(hwndMle,szFilename);
          }
          break;
        default:
          return WinDefWindowProc( hwndParent, msg, mp1, mp2 );
      }

      break;
      }

    default:
      return WinDefWindowProc( hwndParent, msg, mp1, mp2 );
  }
  return (MRESULT)FALSE;
} /* End of EditWindowProc */


/*****************************************************************************
* Convenience function to populate MultiLineEdit with file contents.
*****************************************************************************/
static VOID LoadMLEFromFile(HWND hwndMle, PCHAR pszFileName)
{
  CHAR  szMleBuf[512];
  IPT   lOffset = 0;
  PSZ   pszTextFile;
  HFILE hf;
  ULONG cbCopied;
  ULONG ulAction;
  ULONG cbBytesRead;
  ULONG ulTextLength;

  /* Open the file */
  DosOpen(pszFileName,
          &hf,
          &ulAction,
          0,
          FILE_NORMAL,
          FILE_OPEN |
          FILE_CREATE,
          OPEN_ACCESS_READONLY |
          OPEN_SHARE_DENYNONE,
          NULL);

  /* Zero-fill the buffer using memset, a C run-time function */
  memset(szMleBuf, 0, sizeof(szMleBuf));

  /* Set the MLE import-export buffer */
  WinSendMsg(hwndMle,
             MLM_SETIMPORTEXPORT,
             MPFROMP(szMleBuf),
             MPFROMSHORT((USHORT) sizeof(szMleBuf)));

  /* Don't refresh the MLE while loading */
  WinSendMsg(hwndMle, MLM_DISABLEREFRESH, (MPARAM)0L, (MPARAM)0L);

  /* Get the length of the text in the buffer */
  ulTextLength = LONGFROMMR(WinSendMsg(hwndMle, MLM_QUERYTEXTLENGTH, (MPARAM)0L, (MPARAM)0L));

  /* Select all of the text */
  WinSendMsg(hwndMle, MLM_SETSEL, (MPARAM)0L, (MPARAM)ulTextLength);

  /* Clear the MLE */
  WinSendMsg(hwndMle, MLM_CLEAR, (MPARAM)0L, (MPARAM)0L);

  /**********************************************************************/
  /*  Read the text from the file to the buffer,                        */
  /*  then import it to the MLE.                                        */
  /**********************************************************************/

  do {
       DosRead(hf,
               szMleBuf,
               sizeof(szMleBuf),
               &cbBytesRead);

       cbCopied = (ULONG) WinSendMsg(hwndMle,
                          MLM_IMPORT,
                          MPFROMP( &lOffset),
                          MPFROMP(&cbBytesRead));
     } while (cbBytesRead);

  /* Close the file */
  DosClose(hf);

  /* Allow the MLE to be refreshed again */
  WinSendMsg(hwndMle, MLM_ENABLEREFRESH, (MPARAM)0L, (MPARAM)0L);

}

/*****************************************************************************
* Convenience function to save MultiLineEdit contents to a file.
*****************************************************************************/
static VOID SaveMLEToFile(HWND hwndMle, PCHAR pszFileName)
{
  CHAR  szMleBuf[512];
  IPT   lOffset = 0;
  PSZ   pszTextFile;
  HFILE hf;
  ULONG cbChunk;
  ULONG cbCopy;
  ULONG cbCopied;
  ULONG ulAction;
  ULONG cbBytesWritten;
  ULONG ulTextLength;

  /* Open the file */
  DosOpen(pszFileName,
          &hf,
          &ulAction,
          0,
          FILE_NORMAL,
          FILE_OPEN |
          FILE_CREATE,
          OPEN_ACCESS_WRITEONLY |
          OPEN_SHARE_DENYNONE,
          NULL);

  /* Zero-fill the buffer using memset */
  memset(szMleBuf, 0, sizeof(szMleBuf));

  /* Set the MLE import-export buffer */
  WinSendMsg(hwndMle,
             MLM_SETIMPORTEXPORT,
             MPFROMP(szMleBuf),
             MPFROMSHORT((USHORT) sizeof(szMleBuf)));

  /* Don't refresh the MLE while saving */
  WinSendMsg(hwndMle, MLM_DISABLEREFRESH, (MPARAM)0L, (MPARAM)0L);

  /* Determine the number of bytes to be copied */
  cbCopy = LONGFROMMR(WinSendMsg(hwndMle, MLM_QUERYFORMATTEXTLENGTH,
                                 MPFROMLONG(lOffset), MPFROMLONG(-1)));

  cbCopied =  LONGFROMMR(WinSendMsg(hwndMle, MLM_EXPORT, MPFROMP(&lOffset),
                                      MPFROMP(&cbCopy)));
  DosWrite(hf, szMleBuf, sizeof(szMleBuf), &cbBytesWritten);

  /* Close the file */
  DosClose(hf);

  /* Allow the MLE to be refreshed again */
  WinSendMsg(hwndMle, MLM_ENABLEREFRESH, (MPARAM)0L, (MPARAM)0L);

}

/*****************************************************************************
* Convenience function to display the file dialog for us.
*****************************************************************************/
static BOOL DoFileDialog(HWND hwnd, INT Mode, PCHAR pchFile)
{
   FILEDLG FileDlg;

   memset(&FileDlg, 0, sizeof(FILEDLG));

   FileDlg.cbSize = sizeof(FILEDLG);
   if (Mode == MLE_OPEN)
      FileDlg.fl = FDS_CENTER | FDS_OPEN_DIALOG | FDS_PRELOAD_VOLINFO;
   if (Mode == MLE_SAVE)
      FileDlg.fl = FDS_CENTER | FDS_SAVEAS_DIALOG | FDS_PRELOAD_VOLINFO;

   if (WinFileDlg(HWND_DESKTOP, hwnd, &FileDlg) != DID_OK)
   {
      WinAlarm(HWND_DESKTOP, WA_ERROR);
      return FALSE;
   }

   strcpy(pchFile, FileDlg.szFullFile);
   return TRUE;
}

/*****************************************************************************
* Convenience function to remove MultiLineEdit contents.
*****************************************************************************/
static VOID ClearMLE(HWND hwndMle)
{
  ULONG ulTextLength;

  /* Don't refresh the MLE while loading */
  WinSendMsg(hwndMle, MLM_DISABLEREFRESH, (MPARAM)0L, (MPARAM)0L);

  /* Get the length of the text in the buffer */
  ulTextLength = LONGFROMMR(WinSendMsg(hwndMle, MLM_QUERYTEXTLENGTH, (MPARAM)0L, (MPARAM)0L));

  /* Select all of the text */
  WinSendMsg(hwndMle, MLM_SETSEL, (MPARAM)0L, (MPARAM)ulTextLength);

  /* Clear the MLE */
  WinSendMsg(hwndMle, MLM_CLEAR, (MPARAM)0L, (MPARAM)0L);

  /* Allow the MLE to be refreshed again */
  WinSendMsg(hwndMle, MLM_ENABLEREFRESH, (MPARAM)0L, (MPARAM)0L);

}

