/*****************************************************************************
* OS/2  PM/C Language Demo - MiniPIM
*
*       This demo is intended for comparison with the MiniPIM.java
*       Magercise.
*
* Copyright 1997, <a href=http://www.MageLang.com>MageLang Institute</a>.
******************************************************************************/
#define INCL_WIN
#define INCL_GPI
#define INCL_DOSFILEMGR
#define INCL_DOSERRORS

#include <os2.h>                        /* PM header file                */
#include <string.h>                     /* for strlen prototype          */
#include <stdlib.h>                     /* for calloc prototype          */
#include <stdio.h>                      /* for sprintf prototype         */
#include "PIMPhone.h"                   /* Function prototypes           */

#define NAMELENGTH 32
#define PHONELENGTH 16

/* Local structure for entries */
typedef struct _PhoneEntry PhoneEntry;
struct _PhoneEntry
{
   char name[NAMELENGTH];
   char business[PHONELENGTH];
   char home[PHONELENGTH];
   char fax[PHONELENGTH];
   int removed;
   PhoneEntry *next;
};

/* Convenience function for creating buttons */
static HWND CreateButton(HWND hwndParent, LONG Xpos, LONG Ypos,
                         LONG width, LONG height, PCHAR Caption, ULONG wndID);
static void AddPhoneListItem(PhoneEntry *Entry);
static void RemovePhoneListItem(char *name);
static void UpdatePhoneListWindow(HWND hwnd);
static PhoneEntry *LookupPhoneList(char *name);

static PhoneEntry *Head = NULL;
static PhoneEntry *Tail = NULL;

/* Handles for all of the windows */
HWND hwndStatic1 = NULLHANDLE;          /* Static label 1 window handle  */
HWND hwndStatic2 = NULLHANDLE;          /* Static label 2 window handle  */
HWND hwndStatic3 = NULLHANDLE;          /* Static label 3 window handle  */
HWND hwndStatic4 = NULLHANDLE;          /* Static label 4 window handle  */
HWND hwndComboBox1 = NULLHANDLE;        /* Combo Box 1 window handle     */
HWND hwndEntryField2 = NULLHANDLE;      /* Entry Field 2 window handle   */
HWND hwndEntryField3 = NULLHANDLE;      /* Entry Field 3 window handle   */
HWND hwndEntryField4 = NULLHANDLE;      /* Entry Field 4 window handle   */
static HWND hwndAddBtn;                 /* Add Button window handle      */
static HWND hwndRemBtn;                 /* Remove Button window handle   */

/* Control ID's */
#define ID_STATIC1       380
#define ID_STATIC2       381
#define ID_STATIC3       382
#define ID_STATIC4       383
#define ID_COMBOBOX1     384
#define ID_ENTRYFIELD2   385
#define ID_ENTRYFIELD3   386
#define ID_ENTRYFIELD4   387
#define ID_ADDBUTTON     388
#define ID_REMBUTTON     389

#define SX    10
#define SY    150
#define SW    100
#define SH    20
#define YSP   40
#define EFH   20
#define EFW   300
#define XSP   5

static CHAR szEntry[50];

/*****************************************************/
/* Register the Phone List window class              */
/*                                                   */
/* Keep the details of the class name local to this  */
/* module so we can change them independent from the */
/* main program.                                     */
/*****************************************************/
BOOL RegisterPhoneClass(HAB hab)
{
   return WinRegisterClass(hab, (PSZ)"PIMPhone", (PFNWP)PhoneWindowProc, CS_SIZEREDRAW, 0);
}

/*****************************************************/
/* Create the Phone List window                      */
/*                                                   */
/* The window is created as a child of the parent,   */
/* which is also the owner of this window.           */
/*****************************************************/
HWND CreatePhoneWindow(HWND hwndParent)
{
   HWND hwnd = WinCreateWindow(hwndParent,
                               "PIMPhone",
                               "",
                               0,               /* Initially invisible */
                               0, 0, 495, 400,
                               hwndParent,
                               HWND_TOP,
                               ID_PHONEWINDOW,
                               NULL,
                               NULL);
   return hwnd;
}

/*****************************************************/
/* Phone List Window Proc                            */
/*                                                   */
/* Handle events directed to the phone list window.  */
/*****************************************************/
MRESULT EXPENTRY PhoneWindowProc(HWND hwndParent, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  LONG i;
  RECTL rcl;

  switch( msg )
  {
    case WM_CREATE:

      hwndStatic1 = WinCreateWindow(
                hwndParent,                    /* Parent window          */
                WC_STATIC,                     /* Window class           */
                "Name : ",                     /* Window text            */
                WS_VISIBLE |                   /* Make it visible        */
                SS_TEXT    |                   /* Static-text control    */
                DT_VCENTER |                   /* Center text vert.      */
                DT_RIGHT,                      /* Right justify text     */
                SX,                            /* x position             */
                SY+3*(SH+YSP),                 /* y position             */
                SW,                            /* Width                  */
                SH,                            /* Height                 */
                hwndParent,                    /* Owner window           */
                HWND_TOP,                      /* Top of z-order         */
                ID_STATIC1,                    /* Window identifier      */
                NULL,                          /* Control data           */
                NULL);                         /* Presentation parameters*/

      hwndStatic2 = WinCreateWindow(
                hwndParent,                    /* Parent window          */
                WC_STATIC,                     /* Window class           */
                "Business : ",                 /* Window text            */
                WS_VISIBLE |                   /* Make it visible        */
                SS_TEXT    |                   /* Static-text control    */
                DT_VCENTER |                   /* Center text vert.      */
                DT_RIGHT,                      /* Right justify text     */
                SX,                            /* x position             */
                SY+2*(SH+YSP),                 /* y position             */
                SW,                            /* Width                  */
                SH,                            /* Height                 */
                hwndParent,                    /* Owner window           */
                HWND_TOP,                      /* Top of z-order         */
                ID_STATIC2,                    /* Window identifier      */
                NULL,                          /* Control data           */
                NULL);                         /* Presentation parameters*/

      hwndStatic3 = WinCreateWindow(
                hwndParent,                    /* Parent window          */
                WC_STATIC,                     /* Window class           */
                "Home : ",                     /* Window text            */
                WS_VISIBLE |                   /* Make it visible        */
                SS_TEXT    |                   /* Static-text control    */
                DT_VCENTER |                   /* Center text vert.      */
                DT_RIGHT,                      /* Right justify text     */
                SX,                            /* x position             */
                SY+(SH+YSP),                   /* y position             */
                SW,                            /* Width                  */
                SH,                            /* Height                 */
                hwndParent,                    /* Owner window           */
                HWND_TOP,                      /* Top of z-order         */
                ID_STATIC3,                    /* Window identifier      */
                NULL,                          /* Control data           */
                NULL);                         /* Presentation parameters*/

      hwndStatic4 = WinCreateWindow(
                hwndParent,                    /* Parent window          */
                WC_STATIC,                     /* Window class           */
                "Fax : ",                      /* Window text            */
                WS_VISIBLE |                   /* Make it visible        */
                SS_TEXT    |                   /* Static-text control    */
                DT_VCENTER |                   /* Center text vert.      */
                DT_RIGHT,                      /* Right justify text     */
                SX,                            /* x position             */
                SY,                            /* y position             */
                SW,                            /* Width                  */
                SH,                            /* Height                 */
                hwndParent,                    /* Owner window           */
                HWND_TOP,                      /* Top of z-order         */
                ID_STATIC4,                    /* Window identifier      */
                NULL,                          /* Control data           */
                NULL);                         /* Presentation parameters*/

      hwndComboBox1 = WinCreateWindow(
                hwndParent,                    /* Parent window          */
                WC_COMBOBOX,                   /* Window class           */
                "",                            /* Window text            */
                WS_VISIBLE |                   /* Make it visible        */
                WS_TABSTOP |                   /* Make it "tab"able      */
                CBS_DROPDOWN,                  /* Drop Down Combo Box    */
                SX+SW+XSP,                     /* x position             */
                SY+3*(EFH+YSP)-EFH*5,          /* y position             */
                EFW,                           /* Width                  */
                EFH*6,                         /* Height                 */
                hwndParent,                    /* Owner window           */
                HWND_TOP,                      /* Top of z-order         */
                ID_COMBOBOX1,                  /* Window identifier      */
                NULL,                          /* Control data           */
                NULL);                         /* Presentation parameters*/
      hwndEntryField2 = WinCreateWindow(
                hwndParent,                    /* Parent window          */
                WC_ENTRYFIELD,                 /* Window class           */
                "",                            /* Window text            */
                WS_VISIBLE |                   /* Make it visible        */
                WS_TABSTOP |                   /* Make it "tab"able      */
                ES_AUTOSCROLL |                /* Entry Field Control    */
                ES_MARGIN,                     /* Right Justify horiz.   */
                SX+SW+XSP,                     /* x position             */
                SY+2*(SH+YSP),                 /* y position             */
                EFW,                           /* Width                  */
                EFH,                           /* Height                 */
                hwndParent,                    /* Owner window           */
                HWND_TOP,                      /* Top of z-order         */
                ID_ENTRYFIELD2,                /* Window identifier      */
                NULL,                          /* Control data           */
                NULL);                         /* Presentation parameters*/
      hwndEntryField3 = WinCreateWindow(
                hwndParent,                    /* Parent window          */
                WC_ENTRYFIELD,                 /* Window class           */
                "",                            /* Window text            */
                WS_VISIBLE |                   /* Make it visible        */
                WS_TABSTOP |                   /* Make it "tab"able      */
                ES_AUTOSCROLL |                /* Entry Field Control    */
                ES_MARGIN,                     /* Right Justify horiz.   */
                SX+SW+XSP,                     /* x position             */
                SY+(EFH+YSP),                  /* y position             */
                EFW,                           /* Width                  */
                EFH,                           /* Height                 */
                hwndParent,                    /* Owner window           */
                HWND_TOP,                      /* Top of z-order         */
                ID_ENTRYFIELD3,                /* Window identifier      */
                NULL,                          /* Control data           */
                NULL);                         /* Presentation parameters*/
      hwndEntryField4 = WinCreateWindow(
                hwndParent,                    /* Parent window          */
                WC_ENTRYFIELD,                 /* Window class           */
                "",                            /* Window text            */
                WS_VISIBLE |                   /* Make it visible        */
                WS_TABSTOP |                   /* Make it "tab"able      */
                ES_AUTOSCROLL |                /* Entry Field Control    */
                ES_MARGIN,                     /* Right Justify horiz.   */
                SX+SW+XSP,                     /* x position             */
                SY,                            /* y position             */
                EFW,                           /* Width                  */
                EFH,                           /* Height                 */
                hwndParent,                    /* Owner window           */
                HWND_TOP,                      /* Top of z-order         */
                ID_ENTRYFIELD4,                /* Window identifier      */
                NULL,                          /* Control data           */
                NULL);                         /* Presentation parameters*/
      hwndAddBtn = CreateButton(
                hwndParent,                    /* Parent window          */
                100,                           /* x position             */
                10,                            /* y position             */
                100,                           /* width                  */
                30,                            /* height                 */
                "Add",                         /* Caption                */
                ID_ADDBUTTON);                 /* Window ID              */
      hwndRemBtn = CreateButton(
                hwndParent,                    /* Parent window          */
                300,                           /* x position             */
                10,                            /* y position             */
                100,                           /* width                  */
                30,                            /* height                 */
                "Remove",                      /* Caption                */
                ID_REMBUTTON);                 /* Window ID              */

      UpdatePhoneListWindow(hwndComboBox1);
      break;
    case WM_COMMAND:
      {
         USHORT command;                   /* WM_COMMAND command value     */
         command = SHORT1FROMMP(mp1);      /* Extract the command value    */
         switch (command)
         {
           case ID_ADDBUTTON:
             {
                PhoneEntry Entry;

                memset(&Entry, 0, sizeof(PhoneEntry));

                /* Get data for item to add */
                WinQueryWindowText(hwndComboBox1, NAMELENGTH, Entry.name);
                WinQueryWindowText(hwndEntryField2, PHONELENGTH, Entry.business);
                WinQueryWindowText(hwndEntryField3, PHONELENGTH, Entry.home);
                WinQueryWindowText(hwndEntryField4, PHONELENGTH, Entry.fax);

                /* Save values to linked list */
                AddPhoneListItem(&Entry);
                UpdatePhoneListWindow(hwndComboBox1);
             }
             break;
           case ID_REMBUTTON:
             {
                CHAR Name[NAMELENGTH];

                memset(Name, 0, sizeof(Name));
                /* Get key for item to remove */
                WinQueryWindowText(hwndComboBox1,sizeof(Name), Name);

                /* Remove the Item from the linked list */
                if (Name && strlen(Name))
                   RemovePhoneListItem(Name);

                /* Clear the name from the combo box */
                strcpy(Name, "");
                WinSetWindowText(hwndComboBox1, Name);
                WinSetWindowText(hwndEntryField2, Name);
                WinSetWindowText(hwndEntryField3, Name);
                WinSetWindowText(hwndEntryField4, Name);

                /* Now update the combo box */
                UpdatePhoneListWindow(hwndComboBox1);
             }
             break;
           default:
             return WinDefWindowProc( hwndParent, msg, mp1, mp2 );
         }
      break;
      }
    case WM_CONTROL:
      /*
       * Handle the combo box selection notification
       * message.
       */
      {
      USHORT control;                   /* WM_CONTROL control id        */
      control = SHORT1FROMMP(mp1);      /* Extract the control id       */
      switch (control)
      {
        case ID_COMBOBOX1:
          if (SHORT2FROMMP(mp1) == CBN_EFCHANGE)
          {
             PhoneEntry *temp;
             CHAR Name[NAMELENGTH];

             WinQueryWindowText(hwndComboBox1, NAMELENGTH, Name);
             temp = LookupPhoneList(Name);
             if (temp)
             {
                WinSetWindowText(hwndEntryField2, temp->business);
                WinSetWindowText(hwndEntryField3, temp->home);
                WinSetWindowText(hwndEntryField4, temp->fax);
             }
          }
      }

      break;
      }

    default:
      return WinDefWindowProc( hwndParent, msg, mp1, mp2 );
  }
  return (MRESULT)FALSE;
} /* End of PhoneWindowProc */

/*****************************************************************************
* Convenience function to create buttons.
*****************************************************************************/
static HWND CreateButton(HWND hwndParent, LONG Xpos, LONG Ypos,
                         LONG width, LONG height, PCHAR Caption, ULONG wndID)
{
   HWND hwnd =
   WinCreateWindow(
                hwndParent,                    /* Parent window          */
                WC_BUTTON,                     /* Window class           */
                Caption,                       /* Window text            */
                WS_VISIBLE |
                WS_TABSTOP |                   /* Make it "tab"able      */
                BS_PUSHBUTTON,                 /* Button style           */
                Xpos, 			       /* x position             */
                Ypos,			       /* y position             */
                width,      		       /* Width                  */
                height,                        /* Height                 */
                hwndParent,                    /* Owner window           */
                HWND_TOP,                      /* Top of z-order         */
                wndID,                         /* Window identifier      */
                NULL,                          /* Control data           */
                NULL);                         /* Presentation parameters*/

   return hwnd;
}

/*****************************************************/
/* AddPhoneListItem                                  */
/*                                                   */
/* Create a linked list item to store the phone list */
/* entry and put the data in it.                     */
/*****************************************************/
static void AddPhoneListItem(PhoneEntry *Entry)
{
   PhoneEntry *temp;

   if (Entry && Entry->name)
   {
      /* Remove it if it already exists (low-budget update) */
      RemovePhoneListItem(Entry->name);
      temp = (PhoneEntry *)calloc(1, sizeof(PhoneEntry));
      if (temp)
      {
         /* Add the first item in the list if necessary */
         if (!Head)
             Head = temp;

         /* Add at the end of the list */
         if (Tail)
            Tail->next = temp;

         /* Update the pointer to the last item */
         Tail = temp;

         /* Copy all of the data passed in */
         memcpy(temp, Entry, sizeof(PhoneEntry));
      }
   }
}

/*****************************************************/
/* RemovePhoneListItem                               */
/*                                                   */
/* Mark a linked list item to be removed from the    */
/* phone list and wipe out the key.                  */
/*****************************************************/
static void RemovePhoneListItem(char *name)
{
   PhoneEntry *temp = Head;

   /* Walk through the list looking for the item with the key */
   while (temp && name && strcmp(temp->name, name) != 0)
   {
      temp = temp->next;
   }

   /* If we found an entry, mark it */
   if (temp)
   {
      /* flag as removed */
      temp->removed = TRUE;

      /* Then wipe out the key */
      strcpy(temp->name, "*****");
   }
}

/*****************************************************/
/* Phone List Load Function                          */
/*                                                   */
/* Load the Phone List contents from the ini file    */
/* associated with this application.                 */
/*****************************************************/
void LoadPhoneList(PSZ appname, HINI hini)
{
   SHORT i = 0;
   PhoneEntry Entry;
   BOOL rc = FALSE;
   CHAR key[11];
   ULONG ulSize;

   for (i = 0 ; i < 50 ; i++)
   {
      memset(&Entry, 0, sizeof(PhoneEntry));
      sprintf(key, "Phone%05hd", i);
      ulSize = sizeof(PhoneEntry);
      rc = PrfQueryProfileData(hini, appname, key, &Entry, &ulSize);
      if (rc)
         AddPhoneListItem(&Entry);
   }
}

/*****************************************************/
/* Phone List Save Function                          */
/*                                                   */
/* Save the Phone List contents to the ini file      */
/* associated with this application.                 */
/*****************************************************/
void SavePhoneList(PSZ appname, HINI hini)
{
   SHORT i = 0;
   CHAR key[11];
   PhoneEntry *temp = Head;

   for (i = 0 ; i < 50 && temp ; i++)
   {
      sprintf(key, "Phone%05hd", i);
      if (!temp->removed)
         PrfWriteProfileData(hini, appname, key, temp, (ULONG)sizeof(PhoneEntry));
      temp = temp->next;
   }
}

/*****************************************************/
/* UpdatePhoneListWindow                             */
/*                                                   */
/* Copy the name field in each entry in the Phone    */
/* List to the combobox control used for selection.  */
/*****************************************************/
static void UpdatePhoneListWindow(HWND hwnd)
{
   PhoneEntry *temp = Head;
   INT i = 0;

   /* Disable window refresh while updating */
   WinEnableWindowUpdate(hwnd, FALSE);

   /* Remove all existing entries */
   WinSendMsg(hwnd, LM_DELETEALL, (MPARAM)0, (MPARAM)0);
   for (i = 0 ; i < 50 ; i++)
   {
      /* Add the current list item */
      if (temp && !temp->removed)
         WinSendMsg(hwnd, LM_INSERTITEM, MPFROMSHORT(LIT_END),
                    MPFROMP(temp->name));
      if (temp)
         temp = temp->next;
   }

   /* Enable the window to be refreshed again */
   WinShowWindow(hwnd, TRUE);
}

/*****************************************************/
/* LookupPhoneList                                   */
/*                                                   */
/* Lookup an entry in the linked list given a name.  */
/*****************************************************/
static PhoneEntry *LookupPhoneList(char *name)
{
   PhoneEntry *temp = Head;

   while(temp && (strcmp(temp->name, name) != 0))
      temp = temp->next;

   return temp;
}
