/*****************************************************************************
* OS/2  PM/C Language Demo - MiniPIM
*
*       This demo is intended for comparison with the MiniPIM.java
*       Magercise.
*
* Copyright 1997, <a href=http://www.MageLang.com>MageLang Institute</a>.
******************************************************************************/
#define INCL_WIN
#define INCL_GPI
#define INCL_DOSFILEMGR
#define INCL_DOSERRORS

#include <os2.h>                        /* PM header file               */
#include <string.h>                     /* for strlen prototype         */
#include <stdlib.h>                     /* for calloc prototype          */
#include <stdio.h>                      /* for sprintf prototype        */
#include "PIMToDo.h"                    /* Function prototypes          */

#define TODOLENGTH 100

/* Local structure for entries */
typedef struct _ToDoEntry ToDoEntry;
struct _ToDoEntry
{
   char item[TODOLENGTH];
   int removed;
   ToDoEntry *next;
};

/* Convenience function for creating buttons */
static HWND CreateButton(HWND hwndParent, LONG Xpos, LONG Ypos,
                         LONG width, LONG height, PCHAR Caption, ULONG wndID);
static void AddToDoListItem(ToDoEntry *Entry);
static void RemoveToDoListItem(char *item);
void LoadToDoList(PSZ appname, HINI hini);
void SaveToDoList(PSZ appname, HINI hini);
static void UpdateToDoListWindow(HWND hwnd);

static ToDoEntry *Head = NULL;
static ToDoEntry *Tail = NULL;

/* Handles for all of the windows */
static HWND hwndList;                   /* ListBox window handle        */
static HWND hwndEntry;                  /* EntryField window handle     */
static HWND hwndAddBtn;                 /* Add Button window handle     */
static HWND hwndRemBtn;                 /* Remove Button window handle  */

static CHAR szEntry[50];


/*****************************************************/
/* Register the To-Do List window class              */
/*                                                   */
/* Keep the details of the class name local to this  */
/* module so we can change them independent from the */
/* main program.                                     */
/*****************************************************/
BOOL RegisterToDoClass(HAB hab)
{
   return WinRegisterClass(hab, (PSZ)"PIMToDo", (PFNWP)ToDoWindowProc, CS_SIZEREDRAW, 0);
}

/*****************************************************/
/* Create the To-Do List window                      */
/*                                                   */
/* The window is created as a child of the parent,   */
/* which is also the owner of this window.           */
/*****************************************************/
HWND CreateToDoWindow(HWND hwndParent)
{
   HWND hwnd = WinCreateWindow(hwndParent,
                               "PIMToDo",
                               "",
                               0,                /* Initially invisible */
                               0, 0, 495, 400,
                               hwndParent,
                               HWND_TOP,
                               ID_TODOWINDOW,
                               NULL,
                               NULL);
   return hwnd;
}

/*****************************************************/
/* To-Do List Window Proc                            */
/*                                                   */
/* Handle events directed to the To-Do List window.  */
/*****************************************************/
MRESULT EXPENTRY ToDoWindowProc(HWND hwndParent, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  LONG i;
  RECTL rcl;

  switch( msg )
  {
    case WM_CREATE:

       hwndList = WinCreateWindow(
                 hwndParent,                    /* Parent window          */
                 WC_LISTBOX,                    /* Window class           */
                 "",                            /* Window text            */
                 WS_VISIBLE,                    /* Make it Visible        */
                 5, 			        /* x position             */
                 55,			        /* y position             */
                 480,      	                /* Width                  */
                 300,                           /* Height                 */
                 hwndParent,                    /* Owner window           */
                 HWND_TOP,                      /* Top of z-order         */
                 ID_LIST1,                      /* Window identifier      */
                 NULL,                          /* Control data           */
                 NULL);                         /* Presentation parameters*/
       hwndEntry = WinCreateWindow(
                 hwndParent,                    /* Parent window          */
                 WC_ENTRYFIELD,                 /* Window class           */
                 "",                            /* Window text            */
                 WS_VISIBLE |
                 ES_MARGIN,                     /* Make it Visible        */
                 5, 			        /* x position             */
                 370,			        /* y position             */
                 480,      	                /* Width                  */
                 20,                            /* Height                 */
                 hwndParent,                    /* Owner window           */
                 HWND_TOP,                      /* Top of z-order         */
                 ID_ENTRY1,                     /* Window identifier      */
                 NULL,                          /* Control data           */
                 NULL);                         /* Presentation parameters*/
      hwndAddBtn = CreateButton(
                hwndParent,                     /* Parent window          */
                100, 		 	        /* x position             */
                10,			        /* y position             */
                100, 			        /* width                  */
                30,			        /* height                 */
                "Add",                          /* Caption                */
                ID_ADDBUTTON);                  /* Window ID              */
      hwndRemBtn = CreateButton(
                hwndParent,                     /* Parent window          */
                300, 			        /* x position             */
                10,			        /* y position             */
                100, 			        /* width                  */
                30,			        /* height                 */
                "Remove",                       /* Caption                */
                ID_REMBUTTON);                  /* Window ID              */
      UpdateToDoListWindow(hwndList);
      break;
    case WM_COMMAND:
      {
         USHORT command;                   /* WM_COMMAND command value     */
         command = SHORT1FROMMP(mp1);      /* Extract the command value    */
         switch (command)
         {
           case ID_ADDBUTTON:
             {
                ToDoEntry Entry;

                memset(&Entry, 0, sizeof(ToDoEntry));
                WinQueryWindowText(hwndEntry, TODOLENGTH, Entry.item);

                /* Save values to linked list */
                AddToDoListItem(&Entry);
                UpdateToDoListWindow(hwndList);
             }
             break;
           case ID_REMBUTTON:
             {
                SHORT rc = 0;
                ToDoEntry Entry;

                memset(&Entry, 0, sizeof (ToDoEntry));

                /* Check for a selected item */
                rc = WinQueryLboxSelectedItem(hwndList);

                if (rc != LIT_NONE)
                {
                   WinQueryLboxItemText(hwndList, rc, Entry.item, TODOLENGTH);
                   RemoveToDoListItem(Entry.item);
                   UpdateToDoListWindow(hwndList);
                }
             }
             break;
           default:
             return WinDefWindowProc( hwndParent, msg, mp1, mp2 );
         }
      break;
      }
    default:
      return WinDefWindowProc( hwndParent, msg, mp1, mp2 );
  }
  return (MRESULT)FALSE;
} /* End of ToDoWindowProc */

/*****************************************************************************
* Convenience function to create buttons.
*****************************************************************************/
static HWND CreateButton(HWND hwndParent, LONG Xpos, LONG Ypos,
                         LONG width, LONG height, PCHAR Caption, ULONG wndID)
{
   HWND hwnd =
   WinCreateWindow(
                hwndParent,                    /* Parent window          */
                WC_BUTTON,                     /* Window class           */
                Caption,                       /* Window text            */
                WS_VISIBLE |
                BS_PUSHBUTTON,                 /* Button style           */
                Xpos, 			       /* x position             */
                Ypos,			       /* y position             */
                width,      		       /* Width                  */
                height,                        /* Height                 */
                hwndParent,                    /* Owner window           */
                HWND_TOP,                      /* Top of z-order         */
                wndID,                         /* Window identifier      */
                NULL,                          /* Control data           */
                NULL);                         /* Presentation parameters*/

   return hwnd;
}

/*****************************************************/
/* AddToDoListItem                                   */
/*                                                   */
/* Create a linked list item to store the ToDo list  */
/* entry and put the data in it.                     */
/*****************************************************/
static void AddToDoListItem(ToDoEntry *Entry)
{
   ToDoEntry *temp;

   if (Entry && Entry->item)
   {
      /* Remove it if it already exists (low-budget update) */
      RemoveToDoListItem(Entry->item);
      temp = (ToDoEntry *)calloc(1, sizeof(ToDoEntry));
      if (temp)
      {
         /* Add the first item in the list if necessary */
         if (!Head)
             Head = temp;

         /* Add at the end of the list */
         if (Tail)
            Tail->next = temp;

         /* Update the pointer to the last item */
         Tail = temp;

         /* Copy all of the data passed in */
         memcpy(temp, Entry, sizeof(ToDoEntry));
      }
   }
}

/*****************************************************/
/* RemoveToDoListItem                                */
/*                                                   */
/* Mark a linked list item to be removed from the    */
/* ToDo list and wipe out the key.                   */
/*****************************************************/
static void RemoveToDoListItem(char *item)
{
   ToDoEntry *temp = Head;

   /* Walk through the list looking for the item with the key */
   while (temp && item && strcmp(temp->item, item) != 0)
      temp = temp->next;

   /* If we found an entry, mark it */
   if (temp)
   {
      /* flag as removed */
      temp->removed = TRUE;

      /* Then wipe out the key */
      strcpy(temp->item, "*****");
   }
}

/*****************************************************/
/* ToDo List Load Function                           */
/*                                                   */
/* Load the ToDo List contents from the ini file     */
/* associated with this application.                 */
/*****************************************************/
void LoadToDoList(PSZ appname, HINI hini)
{
   SHORT i = 0;
   ToDoEntry Entry;
   BOOL rc = FALSE;
   CHAR key[10];
   ULONG ulSize;

   for (i = 0 ; i < 50 ; i++)
   {
      memset(&Entry, 0, sizeof(ToDoEntry));
      sprintf(key, "ToDo%05hd", i);
      ulSize = sizeof(ToDoEntry);
      rc = PrfQueryProfileData(hini, appname, key, &Entry, &ulSize);
      if (rc)
         AddToDoListItem(&Entry);
   }
}

/*****************************************************/
/* ToDo List Save Function                          */
/*                                                   */
/* Save the ToDo List contents to the ini file      */
/* associated with this application.                 */
/*****************************************************/
void SaveToDoList(PSZ appname, HINI hini)
{
   SHORT i = 0;
   CHAR key[10];
   ToDoEntry *temp = Head;

   for (i = 0 ; i < 50 && temp ; i++)
   {
      sprintf(key, "ToDo%05hd", i);
      if (!temp->removed)
         PrfWriteProfileData(hini, appname, key, temp, (ULONG)sizeof(ToDoEntry));
      temp = temp->next;
   }
}

/*****************************************************/
/* UpdateToDoListWindow                              */
/*                                                   */
/* Copy the item field in each entry in the ToDo     */
/* List to the listbox control used for display.     */
/*****************************************************/
static void UpdateToDoListWindow(HWND hwnd)
{
   ToDoEntry *temp = Head;
   INT i = 0;

   /* Disable window refresh while updating */
   WinEnableWindowUpdate(hwnd, FALSE);

   /* Remove all existing entries */
   WinSendMsg(hwnd, LM_DELETEALL, (MPARAM)0, (MPARAM)0);
   for (i = 0 ; i < 50 ; i++)
   {
      /* Add the current list item */
      if (temp && !temp->removed)
         WinSendMsg(hwnd, LM_INSERTITEM, MPFROMSHORT(LIT_END),
                    MPFROMP(temp->item));
      if (temp)
         temp = temp->next;
   }

   /* Enable the window to be refreshed again */
   WinShowWindow(hwnd, TRUE);
}


